@@ -15,48 +15,77 @@
 
 daily2annual <-function(x, ...) UseMethod("daily2annual")
 
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: XX-XXX-2008                 #
+# Updates: 09-Aug-2011                 #
+########################################
 daily2annual.default <- function(x, FUN, na.rm=TRUE, out.fmt="%Y",...) {
 
-	 # Checking that 'x' is a zoo object
-	 if (is.na(match(class(x), c("zoo"))))
-	   stop("Invalid argument: 'x' must be of class 'zoo'")
+     # Checking that the user provied a valid class for 'x'   
+     valid.class <- c("xts", "zoo")    
+     if (length(which(!is.na(match(class(x), valid.class )))) <= 0)  
+        stop("Invalid argument: 'class(x)' must be in c('xts', 'zoo')")
 
-	 # Checking that the user provide a valid value for 'FUN'
-	 if (missing(FUN)) {
-		 stop("Missing argument value: 'FUN' must contain a valid function for aggregating the daily values") }
-
-         # Checking that 'x' is a zoo object
-	 if ( is.na(match(out.fmt, c("%Y", "%Y-%m-%d") ) ) )
-	   stop("Invalid argument: 'out.fmt' must be in c('%Y', '%Y-%m-%d')" )
-
-	 # Requiring the Zoo Library (Z’s ordered observations)
-	 require(zoo)
-
-	 # Checking the user provide a valid value for 'x'
-	 if (is.na(match(sfreq(x), c("daily", "monthly")))) {
-	   stop(paste("Invalid argument: 'x' is not a daily or mothly ts, it is a ", sfreq(x), " ts", sep="") ) }
-
-	 # Generating an Annual time series of Total Annual Precipitation (Annual sum of daily values)
-	 tmp <- aggregate(x, by=format( time(x), "%Y" ), FUN, na.rm= na.rm)
-
-	 # Getting the position of all the years in which there were no values
-         # mean(NA:NA, na.rm=TRUE) == NaN
-         nan.index <- which(is.nan(tmp))
-
-         # Changing all the NaN's by NA's
-         if ( length(nan.index) > 0 ) 
-            tmp[nan.index] <- NA
-
-	 # If the user wants a complete data format for the output annual series:
-	 if (out.fmt == "%Y-%m-%d")
-	   time(tmp) <- as.Date(paste( time(tmp), "-01-01", sep=""))
-
-	 return(tmp)
+     # Requiring the Zoo Library (Z’s ordered observations)
+     require(zoo)
 
+     daily2annual.zoo(x=x, FUN=FUN, na.rm=na.rm,...)
+     
 } # 'daily2annual.default' end
 
 
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: 09-Aug-2011                 #
+# Updates: 09-Aug-2011                 #
+########################################
+daily2annual.zoo <- function(x, FUN, na.rm=TRUE, out.fmt="%Y",...) {
 
+	 # Checking that the user provide a valid value for 'FUN'
+	 if (missing(FUN)) 
+           stop("Missing argument value: 'FUN' must contain a valid function for aggregating the daily values")
+           
+          # Checking the user provide a valid value for 'x'
+	 if (is.na(match(sfreq(x), c("daily", "monthly")))) 
+	   stop(paste("Invalid argument: 'x' is not a daily or mothly ts, it is a ", sfreq(x), " ts", sep="") )  
+
+         # Checking 'out.fmt'
+	 if ( is.na(match(out.fmt, c("%Y", "%Y-%m-%d") ) ) )
+	   stop("Invalid argument: 'out.fmt' must be in c('%Y', '%Y-%m-%d')" )	
+	   
+	 # Annual index for 'x'
+         dates  <- time(x)
+         y      <- as.numeric(format( dates, "%Y"))
+         years  <- factor( y, levels=unique(y) )
+
+	 # Computing Annual time series
+	 tmp <- aggregate(x, by=years, FUN, na.rm= na.rm)
+
+	 # Replacing the NaNs by 'NA.
+         # mean(NA:NA, na.rm=TRUE) == NaN
+         nan.index <- which(is.nan(tmp))
+         if ( length(nan.index) > 0 ) { tmp[nan.index] <- NA }
+          
+         # Replacing all the Inf and -Inf by NA's
+         # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
+         inf.index <- which(is.infinite(tmp))
+         if ( length(inf.index) > 0 ) tmp[inf.index] <- NA 
+        
+	 # If the user wants a complete data format for the output annual series:
+	 if (out.fmt == "%Y-%m-%d")
+	    time(tmp) <- as.Date(paste( time(tmp), "-01-01", sep=""))
+
+	 return(tmp)
+
+} # 'daily2annual.zoo' end
+
+
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: XX-XXX-2008                 #
+# Updates: 09-Aug-2011                 #
+########################################
 # 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
 #             dates for correponding to the 'sname' station
 #             If 'dates' is a number, it indicates the index of the column in
@@ -81,14 +110,15 @@ daily2annual.data.frame <- function(x, FUN, na.rm=TRUE, out.fmt="%Y",
                                     dates, date.fmt="%Y-%m-%d",
                                     out.type="data.frame",
                                     verbose=TRUE,...) {
-  # Checking that the user provide a valid value for 'FUN'
-  if (missing(FUN))
-    stop("Missing argument value: 'FUN' must contain a valid function for aggregating the daily values")
-
+                                    
   # Checking that the user provied a valid argument for 'out.type'
   if (is.na(match( out.type, c("data.frame", "db") ) ) )
       stop("Invalid argument: 'out.type' must be in c('data.frame', 'db'")
 
+  # Checking that the user provide a valid value for 'FUN'
+  if (missing(FUN))
+    stop("Missing argument value: 'FUN' must contain a valid function for aggregating the daily values")
+
   # Checking that the user provied a valid argument for 'dates'
   if (missing(dates)) {
       stop("Missing argument: 'dates' must be provided")
@@ -114,112 +144,87 @@ daily2annual.data.frame <- function(x, FUN, na.rm=TRUE, out.fmt="%Y",
   # time series corresponding to the 'st.name' station
   if ( ( class(dates) == "Date") & (length(dates) != nrow(x) ) )
      stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
-
-  # Amount of stations in 'x'
-  nstations <- ncol(x)
-
-  # ID of all the stations in 'x'
-  snames <- colnames(x)
-
-  # Computing the Starting and Ending Year of the analysis
-  Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
-  Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))
-
-  # Amount of Years belonging to the desired period
-  #nyears <- Ending.Year - Starting.Year + 1
-
-  # Computing the amount of years with data within 'x'
-  ndays    <- length(dates) # number of days in the period
-  tmp      <- vector2zoo(rep(0, ndays), dates)
-  tmp      <- daily2annual.default(x= tmp, FUN=FUN, na.rm=na.rm)
-  nyears   <- length(tmp) #number of years in the period
-
-  # Generating a string vector with the years effectively within 'x'
-  if (out.fmt == "%Y") {
-    chryears <- time(tmp)
-  } else chryears <- format(time(tmp), "%Y")
-
-
-  # Requiring the Zoo Library (Z’s ordered observations)
-  require(zoo)
-
-  if (verbose) message("[Starting the computations...]")
-
+     
+  # Transforming 'x' into a zoo object
+  x <- zoo(x, dates)
+  
+  ##############################################################################
   if (out.type == "data.frame") {
+  
+    z <- daily2annual.zoo(x=x, FUN=FUN, na.rm=na.rm, ...)
+    
+  } else if (out.type == "db") { 
 
-	# Vector with the names of the field that will be used for storing the results
-	field.names <- snames
+       if (verbose) message("[Starting the computations...]")
+       
+       # Amount of stations in 'x'
+       nstations <- ncol(x)
 
-	# Creating the data.frame that will store the computed averages for each station
-	z <- as.data.frame(matrix(data = NA, nrow = nyears, ncol = nstations,
-                                  byrow = TRUE, dimnames = NULL) )
+       # ID of all the stations in 'x'
+       snames <- colnames(x)
 
-	colnames(z) <- field.names
-	rownames(z) <- chryears
+       # Computing the Starting and Ending Year of the analysis
+       Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
+       Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))
 
-	z[1:nstations] <- sapply(1:nstations, function(j,y) {
+       # Amount of Years belonging to the desired period
+       #nyears <- Ending.Year - Starting.Year + 1
 
+       # Computing the amount of years with data within 'x'
+       ndays    <- length(dates) # number of days in the period
+       tmp      <- vector2zoo(rep(0, ndays), dates)
+       tmp      <- daily2annual.zoo(x= tmp, FUN=FUN, na.rm=na.rm, out.fmt="%Y-%m-%d")
+       nyears   <- length(tmp) #number of years in the period
 
-	  if (verbose) message( paste("Station: ", format(snames[j], width=10, justify="left"),
-                                      " : ",format(j, width=3, justify="left"), "/",
-                                      nstations, " => ",
-                                      format(round(100*j/nstations,2), width=6, justify="left"),
-                                      "%", sep="") )
+       # Generating a string vector with the years effectively within 'x'
+       if (out.fmt == "%Y") {
+          chryears <- time(tmp)
+       } else chryears <- format(time(tmp), "%Y")
+ 
+       # Creating a vector with the names of the field that will be used for storing the results
+       field.names <- c("StationID", "Year", "Value" )
 
-		# Transforming the column of 'x' into a zoo object,
-		# using the dates provided by the user
-		tmp <- vector2zoo(x=x[,j], dates=dates, date.fmt=date.fmt)
-
-		# Computing the annual values
-		z[,j] <- daily2annual.default(x= tmp, FUN=FUN, na.rm=na.rm)
-
-	}, y = x) # sapply END
-
-  } else if (out.type == "db") {
-
-        # Creating a vector with the names of the field that will be used for storing the results
-        field.names <- c("StationID", "Year", "Value" )
-
-        # Creating the data.frame that will store the computed averages for each subcatchment
-        z <- as.data.frame(matrix(data = NA, nrow = nyears*nstations, ncol = 3,
+       # Creating the data.frame that will store the computed averages for each subcatchment
+       z <- as.data.frame(matrix(data = NA, nrow = nyears*nstations, ncol = 3,
                            byrow = TRUE, dimnames = NULL) )
-        colnames(z) <- field.names
+       colnames(z) <- field.names
 
-        y = x
+       y = x
 
-        for (j in 1:nstations) {
+       for (j in 1:nstations) {
 
-            if (verbose) message( paste("Station: ", format(snames[j], width=10, justify="left"),
+           if (verbose) message( paste("Station: ", format(snames[j], width=10, justify="left"),
                                       " : ",format(j, width=3, justify="left"), "/",
                                       nstations, " => ",
                                       format(round(100*j/nstations,2), width=6, justify="left"),
                                       "%", sep="") )
 
-      # Transforming the column of 'x' into a zoo object,
-	    # using the dates provided by the user
-	    tmp <- vector2zoo(x=x[,j], dates=dates, date.fmt=date.fmt)
+          # Computing the annual values
+          a <- daily2annual.zoo(x= x[,j], FUN=FUN, na.rm=na.rm, out.fmt="%Y-%m-%d")
 
-	    # Computing the annual values
-	    a <- daily2annual.default(x= tmp, FUN=FUN, na.rm=na.rm, out.fmt="%Y-%m-%d")
+          # Putting the annual/monthly values in the output data.frame
+          # The first column of 'x' corresponds to the Year
+          row.ini <- (j-1)*nyears + 1
+          row.fin <-  j*nyears
 
-	    # Putting the annual/monthly values in the output data.frame
-            # The first column of 'x' corresponds to the Year
-            row.ini <- (j-1)*nyears + 1
-            row.fin <-  j*nyears
+          z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 'nmonths' times
+          z[row.ini:row.fin, 2] <- format(as.Date(time(a)), "%Y")
+          z[row.ini:row.fin, 3] <- a
 
-            z[row.ini:row.fin, 1] <- snames[j] # it is automatically repeted 'nmonths' times
-            z[row.ini:row.fin, 2] <- format(as.Date(time(a)), "%Y")
-            z[row.ini:row.fin, 3] <- a
+      } # FOR end
 
-        } # FOR end
-
-    } # IF end
+    } # ELSE end
 
   return( z )
 
  } #'daily2annual.data.frame' END
 
 
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: XX-XXX-2008                 #
+# Updates: 09-Aug-2011                 #
+########################################
 daily2annual.matrix  <- function(x, FUN, na.rm=TRUE, out.fmt="%Y",
                                  dates, date.fmt="%Y-%m-%d",
                                  out.type="data.frame",
